function [ best_beta_param, best_ak_param, perform_mat, perform_cv] = CrossValidation2Param_highorder_syndata...
    ( X, Y, obj_func_str, obj_func_opts, W_org, beta_org, param_range1, param_range2, top_num,top_num_b, cv_fold, eval_func_str, higher_better)
%% Function CROSSVALIDATION1PARAM 
%   Model selection (cross validation) for 1 parameter 
%       multi-task learning functions
%   For usage see test_script.m
%
%% INPUT
%   X:             input data
%   Y:             output data
%   obj_func_str:  1-parameter optimization algorithms 
%   param_range:   the range of the parameter. array
%   cv_fold:       cross validation fold
%   eval_func_str: evaluation function: 
%       signature [performance_measure] = eval_func(Y_test, X_test, W_learnt)
%   higher_better: if the performance is better given
%           higher measurement (e.g., Accuracy, AUC)
%% OUTPUT
%   best_param:  best parameter in the given parameter range 
%   perform_mat: the average performance for every parameter in the
%                parameter range. 
%
%% LICENSE
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%   Copyright (C) 2011 - 2012 Jiayu Zhou and Jieping Ye 
%
%   You are suggested to first read the Manual.
%   For any problem, please contact with Jiayu Zhou via jiayu.zhou@asu.edu
%
%   Last modified on June 3, 2012.
%
  
eval_func = str2func(eval_func_str);
obj_func  = str2func(obj_func_str);


% compute sample size for each task
task_num = length(X);

% performance vector
perform_mat = zeros(length(param_range1),length(param_range2));
perform_cv = zeros(cv_fold,1);

% begin cross validation
fprintf('[')
for cv_idx = 1: cv_fold
    fprintf('.')
    
    perform_temp = 0;
    % buid cross validation data splittings for each task.
    cv_Xtr = cell(task_num, 1);
    cv_Ytr = cell(task_num, 1);
    cv_Xte = cell(task_num, 1);
    cv_Yte = cell(task_num, 1);
    
    for t = 1: task_num
        task_sample_size = length(Y{t});
        te_idx = cv_idx : cv_fold : task_sample_size;
        tr_idx = setdiff(1:task_sample_size, te_idx);
        
        cv_Xtr{t} = X{t}(tr_idx, :);
        cv_Ytr{t} = Y{t}(tr_idx, :);
        cv_Xte{t} = X{t}(te_idx, :);
        cv_Yte{t} = Y{t}(te_idx, :);
    end
    
    for p_idx1 = 1: length(param_range1)
        perform_temp = 0;
        for p_idx2 = 1: length(param_range2)
            [W,beta] = obj_func(cv_Xtr, cv_Ytr, cv_Xte, cv_Yte, W_org, beta_org, param_range1(p_idx1), param_range2(p_idx2),top_num,top_num_b, obj_func_opts);
            perform_mat(p_idx1,p_idx2) = perform_mat(p_idx1,p_idx2) + eval_func(Y, X, W, beta);
            perform_temp = perform_temp + eval_func(Y, X, W, beta);
        end
    end
    perform_cv(cv_idx) = perform_temp./length(param_range1);
end
perform_mat = perform_mat./cv_fold;
fprintf(']\n')
    
if( higher_better)
    [idx_beta,idx_ak,minvalue]=find(perform_mat==max(perform_mat(:)));
else
    [idx_beta,idx_ak,minvalue]=find(perform_mat==min(perform_mat(:)));
end 

best_beta_param = param_range1(idx_beta(1)); 
best_ak_param = param_range2(idx_ak(1));


end

