%%%%%% Demo for FHIM Logistic Regression on RCC Data %%%%%%
%%%%
%%%% Created by Sanjay at NECLA, Princeton
%%%% Date : July 2014 
%%%% Version: 1.0

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % This program runs Orthant-Wise learning algorithm on RCC dataset. First, it loads RCC data using RCCdata_generate_logreg.m. Then it calls FHIM_Logreg_RCC_CV.m to generate training and test data and performs Cross-Validation to find the best tuning parameters (lambdas). It outputs the ROC score on the test data. 
% % % % Inputs: Interactive inputs -> path_name, case_number. path_name is the full path of the location of the RCC dataset. path_name is a string, for eg. '/home/ml/spurushotham/FHIM Codes/RCC'. case_number is the classification experiment number for RCC dataset. If case_number = 1, then it is benign vs stage 1-4 classification experiment. Similarly, case_number =2 is benign, stage 1 vs stage 2-3 and case_number =3 is benign, stage 1,2 vs stage 3,4. maximum simulations is number of times the classification experiment needs to be run.
% % % % Outputs: Avg. ROC score for the experiments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
clc;

%% Load RCC Dataset
path_name = input('Enter the full path name (string) of the RCC dataset\n');
case_number = input('\nEnter Experiment case number (1 to 3):');
fprintf('\nEntered path name is \n : %s',path_name);
fprintf('\nEntered experiment case : %d ',case_number);
fprintf('\n');
[X, Y, X_original, Y_original] = RCCdata_generate_logreg(path_name, case_number);

max_simulations = input('\nEnter number of simulations :');

%% Run FHIM for logistic regression
CV_fold =5;
param = 1;
split_percent = 0.8;
for sim=1:max_simulations
   [roc_score, W_est, beta_est, ak_est, K_rank] = FHIM_Logreg_RCC_CV(X, Y, split_percent, CV_fold, param);
    roc_perform(sim) = roc_score;
end

fprintf('Avg. ROC score for %d simulations is %.4f\n',max_simulations,mean(roc_perform));
