%%%%%% Demo for FHIM Logistic Regression on RCC Data %%%%%%
%%%%
%%%% Created by Sanjay at NECLA, Princeton
%%%% Date : July 2014
%%%% Version: 1.0

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % This program runs Orthant-Wise learning algorithm on RCC dataset. First, it loads RCC data using RCCdata_generate_logreg.m. Then it calls FHIM_Logreg_RCC_CV.m to generate training and test data and performs Cross-Validation to find the best tuning parameters (lambdas). It outputs the ROC score on the test data.
% % % % Inputs: Interactive inputs -> path_name, case_number. path_name is the full path of the location of the RCC dataset. path_name is a string, for eg. '/home/ml/spurushotham/FHIM Codes/RCC'. case_number is the classification experiment number for RCC dataset. If case_number = 1, then it is benign vs stage 1-4 classification experiment. Similarly, case_number =2 is benign, stage 1 vs stage 2-3 and case_number =3 is benign, stage 1,2 vs stage 3,4. maximum simulations is number of times the classification experiment needs to be run.
% % % % Outputs: Avg. ROC score for the experiments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
clc;

addpath('../common/');
addpath('../../L1LogReg/');

%% Load RCC Dataset
% path_name = input('Enter the full path name (string) of the RCC dataset\n');
path_name = '..';
case_number = input('\nEnter Experiment case number (1 to 3):');
fprintf('\nEntered path name is \n : %s',path_name);
fprintf('\nEntered experiment case : %d ',case_number);
fprintf('\n');
[X, Y, X_original, Y_original] = RCCdata_generate_logreg(path_name, case_number);

max_simulations = input('\nEnter number of simulations :');

%% Run FHIM for Highorder Logistic regression

alpha = 5;
param1 = 0.05;
param2 = [alpha*param1, 2*alpha*param1];

split_percent = 0.8;
CV_fold =5;

if length(param1) >1 || length(param2) >1
    flag_CV = 1; %%% Flag for performing Cross Validation
else
    flag_CV = 0;
end

%% RUN Simulations
for sim=1:max_simulations
    if flag_CV ==0
        [MSE_performance, ROC_performance, W_est, beta_est,ak_est,K_rank_est,beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_RCC_noCV_new_up(X,Y,split_percent,param1, param2);
        MSE_AltOpt(sim) = MSE_performance;
        ROC_AltOpt(sim) = ROC_performance;
        Err_init(sim) = err_init;
        W_es{sim} = W_est;
        beta_es{sim} = beta_est;
        beta_init_es{sim} = beta_init;
        ak_es{sim} = ak_est;
        Krank_es(sim) = K_rank_est;
        ypred_init = cell2mat(X_te)*beta_init;
        ypred_init = (ypred_init>0.5)*2-1;
        roc_init(sim) = roc(ypred_init,cell2mat(Y_te),'no_figure');
        roc_stc1(sim) = 1-roc(X_te{1}(:,1029),cell2mat(Y_te),'no_figure');
        
    else
        %         [MSE_performance, ROC_performance, W_est, beta_est,ak_est,K_rank_est, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_RCC_CV(X,Y,split_percent,param);
        [MSE_performance, ROC_performance, W_est, beta_est, ak_est,K_rank_est, beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_RCC_CV_new_up(X,Y,split_percent, CV_fold, param1, param2);
        MSE_AltOpt(sim) = MSE_performance;
        ROC_AltOpt(sim) = ROC_performance;
        Err_init(sim) = err_init;
        W_es{sim} = W_est;
        beta_es{sim} = beta_est;
        beta_init_es{sim} = beta_init;
        ak_es{sim} = ak_est;
        Krank_es(sim) = K_rank_est;
        ypred_init = cell2mat(X_te)*beta_init;
        ypred_init = (ypred_init>0.5)*2-1;
        roc_init(sim) = roc(ypred_init,cell2mat(Y_te),'no_figure');
        roc_stc1(sim) = 1-roc(X_te{1}(:,1029),cell2mat(Y_te),'no_figure');
    end
end

fprintf('**********************\n');
fprintf('Avg. MSE for %d simulations is %.4f\n',max_simulations,mean(MSE_AltOpt));
fprintf('Error at beta init for %d simulations is %.4f\n', max_simulations, mean(Err_init));
fprintf('Avg. ROC for %d simulations is %.4f\n',max_simulations,mean(ROC_AltOpt));
fprintf('Avg. ROC of STC1 for %d simulations is %.4f \n',max_simulations, mean(roc_stc1));
fprintf('Avg. ROC at init. for %d simulations is %.4f\n',max_simulations,mean(roc_init));
fprintf('**********************\n');

fprintf('NNZ of beta init = %d, NNZ of beta est = %d, NNZ of W_est is %d\n',nnz(beta_init), nnz(beta_est), nnz(W_est));

[val_init,val_idx] =sort(abs(beta_init),'descend');
[val_beta_est,beta_est_idx] =sort(abs(beta_est),'descend');

% % val_idx(1:5)'
% % beta_est_idx(1:5)'

% % % % CV_fold =5;
% % % % param = 1;
% % % % split_percent = 0.8;
% % % % for sim=1:max_simulations
% % % %    [roc_score, W_est, beta_est, ak_est, K_rank] = FHIM_Logreg_RCC_CV(X, Y, split_percent, CV_fold, param);
% % % %     roc_perform(sim) = roc_score;
% % % % end
% % % %
% % % % fprintf('Avg. ROC score for %d simulations is %.4f\n',max_simulations,mean(roc_perform));

