%%%%%% Demo for FHIM Logistic Regression on Synthetic Data %%%%%%
%%%%
%%%% Created by Sanjay at NECLA, Princeton
%%%% Date : July 2014
%%%% Version: 1.0
% Modified by Yizhe Zhang at NEC Labs, Princeton, 6,10,2016 (Include sk, and minor changes)
% 6,21,2016 (Add multinomial regression support)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % % Same as Demo_FHIM_logreg.m , but also plots support recovery of beta and W
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;
clc;


addpath('../common/');
addpath('../../L1LogReg/');

%syn_sim_max = input('\n Enter the number of simulations : ');
syn_sim_max = 1;
%nInstances = input('Enter number of Instances - n : ');
%pVars = input('Enter number of Variables - p : ');
%K_interactions = input('Enter rank of W (choose small integer like [1,5]) : ');
%beta_sparsity = input('Enter number of non-zeros for beta (choose 5-10% of p) : ');
%ak_sparsity = input('Enter number of non-zeros for ak (choose 5-10% of p): ');

tic
for syn_sim=1:syn_sim_max
    
    
    s = RandStream('mt19937ar','Seed',syn_sim);
    RandStream.setGlobalStream(s);
    
    %% Synthetic data
         nInstances = 1000;
         nClass = 3;
         pVars = 50;
         K_interactions = 2;
         beta_sparsity = 10;
         ak_sparsity = 10;
         max_simulations = 1;   
         setting.isPosDef = false;
    
    [X, Y, beta_org, W_org, sk, epsilon] = syndata_generate_logreg_multi(nInstances, nClass, pVars, K_interactions, beta_sparsity, ak_sparsity, setting);
    %b = glmfit(X(:,2:end),Y','binomial','link','logit');    scatter(b',beta_org);
    top_num = ak_sparsity*ak_sparsity*K_interactions; % top_num = 0.05*pVars; assuming 5% sparse
    top_num_b = beta_sparsity;
    
    %% Run FHIM for Highorder Logistic regression
%     param1 = [0.01, 0.05, 0.1];
%     param2 = [0.05, 0.05, 0.1, 0.5];
    
%     param1 = [0.04];
%     param2 = [0.082];
    param1 = [0.001];
    param2 = [0.001];
    
    split_percent = 0.8;
    CV_fold =5;
    

    for sim=1:max_simulations
        %                 [MSE_performance, ROC_performance, W_est, beta_est,ak_est,K_rank_est,beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_Syndata_noCV(X,Y,split_percent,param);
        [MSE_performance, ROC_performance, W_est, beta_est,ak_est,sk_est, K_rank_est,beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_Syndata_noCV_OD_multi(X,Y,  W_org, beta_org,split_percent,top_num,top_num_b,param1, param2, setting);
        MSE_AltOpt(sim) = MSE_performance;
        ROC_AltOpt(sim) = ROC_performance;
        Err_init(sim) = err_init;
        W_es{sim} = W_est;
        beta_es{sim} = beta_est;
        beta_init_es{sim} = beta_init;
        ak_es{sim} = ak_est;
        sk_es{sim} = sk_est;
        Krank_es(sim) = K_rank_est;
        ypred_init = cell2mat(X_te)*beta_init;
        ypred_init = (ypred_init>0.5)*2-1;
        roc_init(sim) = roc(ypred_init,cell2mat(Y_te),'no_figure');
    end
    
    fprintf('**********************\n');
    fprintf('Avg. MSE for %d simulations is %.4f\n',max_simulations,full(mean(MSE_AltOpt)));
    fprintf('Error at beta init for %d simulations is %.4f\n', max_simulations, full(mean(Err_init)));
    fprintf('Avg. ROC for Alt.Opt. %d simulations is %.4f\n',max_simulations,full(mean(ROC_AltOpt)));
    fprintf('Avg. ROC at init. for %d simulations is %.4f\n',max_simulations,full(mean(roc_init)));
    fprintf('**********************\n');
    [F1_W2,Pr_W2,Re_W2,F1_beta2,Pr_beta2,Re_beta2,Krank_top] = plot_supp_rcvy(W_est,beta_est,W_org,beta_org,top_num,top_num_b);
    fprintf('F1_W = %.4f\n',F1_W2);
    fprintf('F1_beta = %.4f\n',F1_beta2);
    
    %% Support recovery of beta initialized
    beta_initial = beta_init;
    beta_lasso = beta_init;
    beta_lasso_temp=sort(abs(beta_init),'descend');
    beta_lasso_top=(abs(beta_init)>=beta_lasso_temp(top_num_b)).*beta_init;
    overlap_borg_blasso = intersect(find(beta_lasso_top~=0)',find(beta_org~=0));
    beta_init = beta_lasso_top;
    
    Pr_betalasso = length(overlap_borg_blasso)/sum(nnz(beta_init)+(1e-10));
    Re_betalasso = length(overlap_borg_blasso)/sum(nnz(beta_org)+(1e-10));
    F1_betalasso = 2*Pr_betalasso*Re_betalasso/(Pr_betalasso+Re_betalasso+(1e-10));
    
    fprintf('F1_beta_init = %.4f\n',F1_betalasso);
    fprintf('NNZ of beta init = %d, NNZ of beta est = %d, NNZ of W_est is %d\n',nnz(beta_initial), nnz(beta_est), nnz(W_est));
    %close all
    
    F1_W(syn_sim) = F1_W2;
    F1_init(syn_sim) = F1_betalasso;
    F1_beta(syn_sim) = F1_beta2;
    
end

fprintf('Mean F1 W est = %.4f, mean F1 beta est = %.4f, mean F1 beta init = %.4f\n', mean(F1_W), mean(F1_beta),mean(F1_init));
toc