function [test_performance_roc, W_est, beta_est, ak_est, K_rank] = FHIM_Logreg_RCC_CV(X, Y, split_percent, CV_fold, param)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function for FHIM for Logistic Regression with high order feature
%% interactions
% %%%%%%%%%%%%%%%
% Inputs:
%        X - Design matrix (n x p)
%        Y - Instances (n x 1)
%        split_percent - Splitting data into train and test sets. Range [0,1]. Default: 0.8
%        CV_fold - Folds for Cross-Validation, Default : 5
%        param - lambda tuning parameter list. Example: param = [ 0.1,
%        0.5, 1, 5, 10]
%
% Outputs:
%        roc_score -  ROC score on the test data
%        W_est - Estimated Weight matrix (pxp) of rank K
%        beta_est - Estimated beta vector (px1) 
%        ak_est - Estimated ak vector, used to form W_est
%        K_rank - Estimated low rank K of the W_est 
% %%%%%%%%%%%%%%%%
%%  Related functions:
%  mtSplitPerc, CrossValidation1Param_highorder, OWD_LogisticReg, eval_MTL_logistic_roc_highorder
%%%%%%%%%%%%%%%%%%%%%%
%  Written by Sanjay at NECLA, Princeton
%  Version : 1.0
%  Date : July 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

train_percent = split_percent;
cv_fold = CV_fold;

% optimization options
opts = [];
opts.maxIter = 100;
opts.verbose = 0;

% Evaluation function - ROC for high order with bias terms
eval_func_str = 'eval_MTL_logistic_roc_highorder';
higher_better = true; %% ROC, AUC, Accuracy - true, MSE - false
% % eval_func_str = 'eval_MTL_logregerror_highorder_new';
% % higher_better = false; %% ROC, AUC, Accuracy - true, Classification Error,MSE - false

%% split data into training and testing, training_percent = 80% i.e 0.8
[X_tr, Y_tr, X_te, Y_te] = mtSplitPerc(X,Y,train_percent);

%% lambda parameter range
param_range = param;

%% Perform cross-validation to find best parameters (best lambdas)
fprintf('Perform model selection via cross validation: \n');
[best_param, perform_mat] = CrossValidation1Param_highorder(X_tr, Y_tr, 'OWD_LogisticReg', opts, param_range, cv_fold, eval_func_str, higher_better);

% Build model using the optimal parameter
[W_est, beta_est, ak_est, K_rank] = OWD_LogisticReg(X_tr, Y_tr, best_param, opts); % Orthant Wise Learning algorithm

% Performance evaluation on test set
test_performance_roc = eval_MTL_logistic_roc_highorder(Y_te, X_te, W_est, beta_est);
% test_performance_error = eval_MTL_logregerror_highorder_new(Y_te, X_te, W_est, beta_est);

end

