% function [test_performance_roc, W_est, beta_est, ak_est, K_rank] = FHIM_Logreg_RCC_noCV(X, Y, split_percent, CV_fold, param)
function [Error_performance, ROC_performance, W_est, beta_est, ak_est,K_rank_est, beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_RCC_CV_new_up(X,Y,split_percent, cv_fold, param1, param2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function for FHIM for Logistic Regression with high order feature
%% interactions
% %%%%%%%%%%%%%%%
% Inputs:
%        X - Design matrix (n x p)
%        Y - Instances (n x 1)
%        split_percent - Splitting data into train and test sets. Range [0,1]. Default: 0.8
%        CV_fold - Folds for Cross-Validation, Default : 5
%        param - lambda tuning parameter list. Example: param = [ 0.1,
%        0.5, 1, 5, 10]
%
% Outputs:
%        roc_score -  ROC score on the test data
%        W_est - Estimated Weight matrix (pxp) of rank K
%        beta_est - Estimated beta vector (px1) 
%        ak_est - Estimated ak vector, used to form W_est
%        K_rank - Estimated low rank K of the W_est 
% %%%%%%%%%%%%%%%%
%%  Related functions:
%  mtSplitPerc, CrossValidation1Param_highorder, OWD_LogisticReg, eval_MTL_logistic_roc_highorder
%%%%%%%%%%%%%%%%%%%%%%
%  Written by Sanjay at NECLA, Princeton
%  Version : 1.0
%  Date : July 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

train_percent = split_percent;

beta_param = param1; % model parameter
ak_param = param2; % model parameter

% optimization options
opts = [];
opts.maxIter = 100;
opts.verbose = 0;

% Evaluation function - ROC for high order with bias terms
eval_func_str = 'eval_MTL_logistic_roc_highorder';
higher_better = true; %% ROC, AUC, Accuracy - true, MSE - false
% % eval_func_str = 'eval_MTL_logregerror_highorder_new';
% % higher_better = false; %% ROC, AUC, Accuracy - true, Classification Error,MSE - false

%% split data into training and testing, training_percent = 80% i.e 0.8
[X_tr, Y_tr, X_te, Y_te] = mtSplitPerc(X,Y,train_percent);

%% lambda parameter range
param_range1 = param1;
param_range2 = param2;

%% Perform cross-validation to find best parameters (best lambdas)
fprintf('Perform model selection via cross validation: \n');
[beta_param, ak_param, perform_mat] = CrossValidation2Param_highorder(X_tr, Y_tr, 'OWD_LogisticReg_OD_RCC', opts, param_range1, param_range2, cv_fold, eval_func_str, higher_better);

% Build model using the optimal parameter
% [W_est, beta_est, ak_est,K_rank_est,beta_init,err_init] = OWD_LogisticReg(X_tr, Y_tr, X_te, Y_te, best_param, opts);
% [W_est, beta_est, ak_est,K_rank_est,beta_init,err_init] = OWD_LogisticReg_OD(X_tr, Y_tr, X_te, Y_te, best_param, opts);
[W_est, beta_est, ak_est,K_rank_est,beta_init,err_init] = OWD_LogisticReg_OD_RCC(X_tr, Y_tr, X_te, Y_te, beta_param, ak_param, opts);

% Performance on test set
Error_performance = eval_MTL_logregerror_highorder_new(Y_te, X_te, W_est, beta_est);
ROC_performance = eval_MTL_logistic_roc_highorder(Y_te, X_te, W_est, beta_est); %% If ROC is used for evaluation

end

