function [Error_performance,ROC_performance, W_est, beta_est, ak_est,K_rank_est, beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_Syndata_CV(X,Y, W_org, beta_org, split_percent, CV_fold,top_num,top_num_b, param1, param2)
%% Function for FHIM for Logistic Regression with high order feature interactions
% %%%%%%%%%%%%%%%
% Inputs:
%        X - Design matrix (n x p)
%        Y - Instances (n x 1)
%        split_percent - Splitting data into train and test sets. Range [0,1]. Default: 0.8
%        CV_fold - Folds for Cross-Validation, Default : 5
%        param - lambda tuning parameter list. Example: param = [ 0.1,
%        0.5, 1, 5, 10]
%
% Outputs:
%       Error_Performance -  Avg. classification error on the test data
%       W_est - Estimated W
%       beta_est - Estimated beta
% %%%%%%%%%%%%%%%%
%  Related functions
%   mtSplitPerc, CrossValidation1Param_highorder_v2, OWD_LogisticReg, eval_MTL_logregerror_highorder_new
%%%%%%%%%%%%%%%%%%%%%%
%  Written by Sanjay at NECLA, Princeton
%  Version : 1.0
%  Date : July 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

X1{1} = X;
Y1{1} = Y';

X = X1;
Y = Y1;
clear X1 Y1

train_percent = split_percent;
cv_fold = CV_fold;

% split data into training and testing.
[X_tr, Y_tr, X_te, Y_te] = mtSplitPerc(X,Y,train_percent);

% the function used for evaluation.
eval_func_str = 'eval_MTL_logregerror_highorder_new';
% % eval_func_str = 'eval_MTL_logistic_roc_highorder'; if roc score is used for evaluation

higher_better = false; %% Accuracy, AUC - true; classification error, mse -  false 
% % higher_better = true; %% Accuracy, AUC - true; classification error -  false 

% optimization options 
opts = [];
opts.maxIter = 100;
opts.verbose = 0;

%% lambda parameter range
param_range1 = param1;
param_range2 = param2;

%% Perform cross-validation to find best parameters (best lambdas)
fprintf('Perform model selection via cross validation: \n');
[beta_param, ak_param, perform_mat] = CrossValidation2Param_highorder_syndata(X_tr, Y_tr, 'OWD_LogisticReg_OD_new_up', opts, W_org, beta_org, param_range1, param_range2, top_num,top_num_b,  cv_fold, eval_func_str, higher_better);

% Build model using the optimal parameter
[W_est, beta_est, ak_est,K_rank_est,beta_init,err_init] = OWD_LogisticReg_OD_new_up(X_tr, Y_tr, X_te, Y_te, W_org, beta_org, beta_param, ak_param, top_num,top_num_b, opts);


% Performance on test set
ROC_performance = eval_MTL_logistic_roc_highorder(Y_te, X_te, W_est, beta_est); %% If ROC is used for evaluation
Error_performance = eval_MTL_logregerror_highorder_new(Y_te, X_te, W_est, beta_est);

end