function [Error_performance, ROC_performance, W_est, beta_est, ak_est,sk_est,K_rank_est, beta_init, err_init, X_tr, Y_tr, X_te, Y_te] = FHIM_Logreg_Syndata_noCV_OD_multi(X,Y,  W_org, beta_org, split_percent, top_num,top_num_b, param1,param2, setting)
%% Function for FHIM for Logistic Regression with high order feature interactions
% %%%%%%%%%%%%%%%
% Inputs:
%        X - Design matrix (n x p)
%        Y - Instances (n x 1)
%        split_percent - Splitting data into train and test sets. Range [0,1]. Default: 0.8
%        CV_fold - Folds for Cross-Validation, Default : 5
%        param - lambda tuning parameter list. Example: param = [ 0.1,
%        0.5, 1, 5, 10]
%
% Outputs:
%       Error_Performance -  Avg. classification error on the test data
%       W_est - Estimated W
%       beta_est - Estimated beta
% %%%%%%%%%%%%%%%%
%  Related functions
%   mtSplitPerc, CrossValidation1Param_highorder_v2, OWD_LogisticReg, eval_MTL_logregerror_highorder_new
%%%%%%%%%%%%%%%%%%%%%%
%  Written by Sanjay at NECLA, Princeton
%  Version : 1.0
%  Date : July 2013
% Modified by Yizhe Zhang at NEC Labs, Princeton, 6,10,2016 (Include sk, and minor changes)
% Modified by Yizhe Zhang at NEC Labs, Princeton, 6,28,2016 (add multinomial regression support)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

X1{1} = X;
Y1{1} = Y';

X = X1;
Y = Y1;
clear X1 Y1

train_percent = split_percent;

beta_param = param1; % model parameter range
ak_param = param2;

% split data into training and testing.
[X_tr, Y_tr, X_te, Y_te] = mtSplitPerc(X,Y,train_percent);

% optimization options 
opts = setting;
opts.maxIter = 100;
opts.verbose = 0;



% build model using the optimal parameter
[W_est, beta_est, ak_est,sk_est ,K_rank_est,beta_init,err_init] = OWD_LogisticReg_OD_multi(X_tr, Y_tr, X_te, Y_te,  W_org, beta_org, beta_param, ak_param, top_num,top_num_b, opts);

% Performance on test set
% % Error_performance = eval_MTL_logistic_roc_highorder(Y_te, X_te, W_est, beta_est); %% If ROC is used for evaluation
Error_performance = eval_MTL_logregerror_highorder_new(Y_te, X_te, W_est, beta_est);
ROC_performance = eval_MTL_logistic_roc_highorder(Y_te, X_te, W_est, beta_est); %% If ROC is used for evaluation

end