function [W,beta,ak,K_rank] = OWD_LogisticReg(X_tr, Y_tr, best_param, opts)
% % % % Orthant-Wise Learning Program for logistic regression model
% % % % Inputs: X_tr, Y_tr, best_param, opts. X_tr, Y_tr ->training data. best_param -> best parameter found from CV. opts -> optimization options declared in the main function
% % % % Outputs: W, beta, ak, K_rank -> They correspond to Estimated W (Weight matrix), estimated beta, estimated ak and estimated K rank.

K_greedy =1;
X = cell2mat(X_tr);
y = cell2mat(Y_tr);

[n,p] = size(X);

% Use Logistic Regression loss function
loss_beta = @LogisticLoss_highorder_beta;
loss_ak = @LogisticLoss_highorder_akak;


% Set all variables to be initially 0
w_init = zeros(p,1);
w_beta = w_init;

Wtemp1 = zeros(p-1,p-1);
for k=1:K_greedy
    w_ak{k} = 1e0*ones(1,p-1)';
    Wtemp1 = Wtemp1 + w_ak{k}'*w_ak{k};
end

options = opts;
options.order = 1;
maxiterations = 250;
%% %%%%%%%%%%%%%%%%%%%% Run Methods %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('\nAlgorithm: Orthant Wise Gradient...\n');
% pause(0.5);

% Set lambda (higher values yield more regularization/sparsity)
lambda_vals = best_param;

% Penalize all variables except bias
lambda_akak = lambda_vals;
lambda_beta = lambda_vals;
lambdaVect_ak = [lambda_vals*ones(p-1,1)];
lambdaVect_beta = [0; lambda_vals*ones(p-1,1)];


while w_ak{K_greedy} ~= 0 & K_greedy <=5
    if K_greedy >2
        w_ak_diff = sum((w_ak{K_greedy-1}- w_ak{K_greedy-2}).^2);
    end
    if K_greedy >1 & (w_ak{K_greedy-1} ==0)
        w_ak{K_greedy} = 0*w_ak{K_greedy};
        break;
    elseif K_greedy>2 & (w_ak_diff < 0.1) %% if the previous is very close to the current w_ak
        w_ak{K_greedy} = 0;
        w_ak{K_greedy-1} = 0;
        break;
    end
    
    % Calculate value of Objective function
    Xbeta = X*w_beta;
        
    for i=1:n
        XWX_pred(i) = X(i,(2:p))*Wtemp1*X(i,(2:p))';
    end
    Xbeta_XWX = (Xbeta+XWX_pred');
    y_pred_temp = 1./(1+exp(-Xbeta_XWX));
    y_pred = (y_pred_temp>=0.5)*2-1;
    ypred_Xbeta_XWX = y.*(Xbeta_XWX);
    
    nll = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta_XWX])); % log loss
    w_ak_penalty = 0;
    for k=1:K_greedy
        w_ak_penalty = w_ak_penalty + lambda_akak*sum(abs(w_ak{k}));
    end
    
    obj_Val = 0;
    obj_Val = nll + lambda_beta*sum(abs(w_beta)) + w_ak_penalty;
    
    % previous and current objective function values
    obj_Val_prev = obj_Val;
    obj_Val_cur = 10^100;
    
    % Some initializations
    iterations = 0;
    criterionObjective = 1;
    tolerance = 1e-12;
    
    while iterations<=maxiterations && criterionObjective > tolerance %&& obj_Val_prev <= obj_Val_cur
        iterations = iterations + 1;
        
        optimFunc_beta = @L1GeneralOrthantWise_higherorder_beta;
        optimFunc_akbk = @L1GeneralOrthantWise_higherorder_akak;
        
% %         Wtemp1 = zeros(p-1,p-1);
% %         for k_iter=1:K_greedy
% %             Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
% %         end
% %         
        for k=1:K_greedy
            % Co-ordinate descent on a_k
            %             tic
            Wtemp1 = zeros(p-1,p-1);
            for k_iter=1:K_greedy
                Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
            end
            
            %             fprintf('\nCo-ordinate descent over a_k at k = %d\n',k);
            lossArgs_ak{k} = {w_beta,X,y,Wtemp1};
            w_ak{k} = optimFunc_akbk(loss_ak, w_ak{k},lambdaVect_ak,options,lossArgs_ak{k}{:});
            %             toc
            if norm(w_ak{k},2) > 0
                w_ak{k} = w_ak{k}/norm(w_ak{k},2);
            end
        end
        
        Wtemp1 = zeros(p-1,p-1);
        for k_iter=1:K_greedy
            Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
        end
        
        % %         Co-ordinate descent on beta
        lossArgs_beta = {X,y,Wtemp1};
        w_beta = optimFunc_beta(loss_beta,w_beta,lambdaVect_beta,options,lossArgs_beta{:});
        % % % %         end
        
        
        clear Xbeta Xbeta_XWX ypred_Xbeta_XWX
        Xbeta = X*w_beta;
        for i=1:n
            XWX_pred(i) = X(i,(2:p))*Wtemp1*X(i,(2:p))';
        end
        Xbeta_XWX = (Xbeta+XWX_pred');
        y_pred_temp = 1./(1+exp(-Xbeta_XWX));
        y_pred = (y_pred_temp>=0.5)*2-1;
        ypred_Xbeta_XWX = y.*(Xbeta_XWX);
        
        nll2 = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta_XWX]));
        
        w_ak_penalty = 0;
        for k=1:K_greedy
            w_ak_penalty = w_ak_penalty + lambda_akak*sum(abs(w_ak{k}));
        end
        obj_Val = nll2 + lambda_beta*sum(abs(w_beta)) + w_ak_penalty;
        obj_Val_cur = obj_Val;
        
        if obj_Val_prev ~= 0;
            criterionObjective = abs(obj_Val_cur-obj_Val_prev)/obj_Val_prev;
        else
            criterionObjective = 0;
        end
        
        fprintf('Curr. obj. val = %12.5e, Prev. obj. val = %12.5e, Criterion = %12.5e, Iteration = %d\n',obj_Val_cur, obj_Val_prev, criterionObjective, iterations);
        obj_Val_prev = obj_Val_cur;
    end
    K_greedy = K_greedy + 1;
    w_ak{K_greedy} = ones(1,p-1)';
end

W = Wtemp1;
beta = w_beta;
ak = w_ak;
K_rank = K_greedy;
