function [W,beta,ak,K_rank,beta_init, err_init] = OWD_LogisticReg_OD_RCC(X_tr, Y_tr, X_te, Y_te, beta_param, ak_param, opts)
% % % % Orthant-Wise Learning Program for logistic regression model
% % % % Inputs: X_tr, Y_tr, best_param, opts. X_tr, Y_tr ->training data. best_param -> best parameter found from CV. opts -> optimization options declared in the main function
% % % % Outputs: W, beta, ak, K_rank -> They correspond to Estimated W (Weight matrix), estimated beta, estimated ak and estimated K rank.

K_greedy =1;
X = cell2mat(X_tr);
y = cell2mat(Y_tr);

[n,p] = size(X);

%% Use High Order Logistic Regression loss function
% loss_beta = @LogisticLoss_highorder_beta_up;
% loss_ak = @LogisticLoss_highorder_akak_up;

loss_beta = @LogisticLoss_highorder_beta_Wnew;
loss_ak = @LogisticLoss_highorder_akak_Wnew;

%% OWL functions for beta and ak updates
% optimFunc_beta = @L1GeneralOrthantWise_higherorder_beta;
% optimFunc_akbk = @L1GeneralOrthantWise_higherorder_akak;

optimFunc_beta = @L1GeneralProjectedSubGradient;
optimFunc_akbk = @L1GeneralProjectedSubGradient;

%% Set lambda (higher values yield more regularization/sparsity)
lambda_beta = beta_param;
lambda_ak = ak_param;

% Penalize all variables except bias
lambda_akak = lambda_ak;
lambda_beta = lambda_beta;
lambdaVect_ak = lambda_akak*ones(p-1,1);
lambdaVect_beta = [0; lambda_beta*ones(p-1,1)];

%% Set all beta variables to be initially 0
w_init = zeros(p,1);
w_beta = w_init;

%% Initialize beta using Lasso or L1LogReg
%% Initialize beta using OLS or Lasso
% % % fprintf('Optimizing beta first using Lasso\n');
% % % % Lasso optimization options
% % % opts1 = [];
% % % opts1.maxIter = 100;
% % % cv_fold1 = 5;
% % % higher_better1 = true;
% % % % model parameter range
% % % param_range1 = [0.01 0.05 0.1 0.5 1 5 10 100];
% % % % the function used for evaluation.
% % % eval_func_str1 = 'eval_MTL_linear_roc';
% % % % Perform cross-validation to find best parameters
% % % fprintf('Perform model selection via cross validation: \n')
% % % [ best_param1 perform_mat1] = CrossValidation1Param...
% % %     ( X_tr, Y_tr, 'Least_Lasso', opts1, param_range1, cv_fold1, eval_func_str1, higher_better1);
% % %
% % % % build model using the optimal parameter
% % % w_beta = Least_Lasso(X_tr, Y_tr, best_param1, opts1);
% % % beta_lasso = w_beta;

lambda_L1logreg = [0.01, 0.1, 1, 10, 20, 30, 50];
for ii = 1:length(lambda_L1logreg)
    [L1LogRegpred{ii}, L1LogRegerr(ii), L1LogRegprob{ii}, L1Logregsw{ii}, L1LogRegbiase{ii}] = L1LogReg_TrnTst(X_tr{1}, Y_tr{1}, X_te{1}, Y_te{1}, ii, 'err');
end
err_L1LogReg = min(L1LogRegerr);
[val,idx] = min(L1LogRegerr);
beta_L1LogReg = L1Logregsw{idx};
w_beta = full(beta_L1LogReg);
beta_init = w_beta;
err_init = err_L1LogReg;

%% Calculate Objective function at initialization
Wtemp1 = zeros(p-1,p-1);
for k=1:K_greedy
    w_ak{k} = 1e0*ones(1,p-1)';
    w_ak{k} = w_ak{k}/norm(w_ak{k},2);
    Wtemp1 = Wtemp1 + w_ak{k}*w_ak{k}';
end
Wtemp1 = Wtemp1 - diag(diag(Wtemp1));

ypred_Xbeta = y.*(X*w_beta);
nll = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta])); % log loss
% nll = nll/n;

w_ak_penalty = 0;
init_obj_Val = 0;
init_obj_Val = nll + lambda_beta*sum(abs(w_beta)) + w_ak_penalty;
init_nll = nll;

nll=0;

clear ypred_Xbeta

fprintf('Loss fun. val. at initialization is %12.5e and Obj. val is %12.5e\n', init_nll, init_obj_Val);
fprintf('***************************************************\n');

Loss_val = [init_nll];
Loss_beta_val = [init_nll];

best_loss_betaOnly = 10^100;
best_loss_ak = 10^100;

%% Some option initializations
options = opts;
options.order = 1;
maxiterations = 25;

%% %%%%%%%%%%%%%%%%%%%% Run Methods %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('\nAlgorithm: Alternating Optimization Using OWL...\n');

while w_ak{K_greedy} ~= 0 & K_greedy <=10
    if K_greedy >2
        w_ak_diff = sum((w_ak{K_greedy-1}- w_ak{K_greedy-2}).^2);
    end
    if K_greedy >1 & (w_ak{K_greedy-1} ==0)
        w_ak{K_greedy} = 0*w_ak{K_greedy};
        K_greedy = K_greedy -2;
        fprintf('previous ak is zero, K_greedy = %d...exiting\n',K_greedy);
        break;
    elseif K_greedy>2 & (w_ak_diff < 0.001) %% if the previous is very close to the current w_ak (w_ak_diff sanity check)
        w_ak{K_greedy} = 0;
        w_ak{K_greedy-1} = 0;
        K_greedy = K_greedy -2;
        fprintf('current ak is close to previous ak...exiting\n');
        break;
    end
    
    %% Calculate value of Objective function
    Wtemp1 = zeros(p-1,p-1);
    for k_iter=1:K_greedy
        Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
    end
    Wtemp1 = Wtemp1-diag(diag(Wtemp1));
    
    Xbeta = X*w_beta;
    for i=1:n
        XWX_pred(i) = X(i,(2:p))*Wtemp1*X(i,(2:p))';
    end
    Xbeta_XWX = (Xbeta+XWX_pred');
    %     y_pred_temp = 1./(1+exp(-Xbeta_XWX));
    %     y_pred = (y_pred_temp>=0.5)*2-1;
    ypred_Xbeta_XWX = y.*(Xbeta_XWX);
    
    nll = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta_XWX])); % log loss
    %     nll = nll/n;
    
    
    clear Xbeta Xbeta_XWX y_pred y_pred_temp ypred_Xbeta_XWX
    w_ak_penalty = 0;
    for k=1:K_greedy
        w_ak_penalty = w_ak_penalty + lambda_akak*sum(abs(w_ak{k}));
    end
    
    obj_Val = 0;
    obj_Val = nll + lambda_beta*sum(abs(w_beta)) + w_ak_penalty;
    %     nll = nll/n;
    
    % previous and current objective function values
    obj_Val_prev = obj_Val;
    obj_Val_cur = 10^100;
    
    % Some initializations
    iterations = 1;
    criterionObjective = 1;
    tolerance = 1e-12;
    curr_loss = nll;
    
    %% Alternating Optimization Loop, 1st optimize ak then beta
    while iterations<=maxiterations && criterionObjective > tolerance %&& obj_Val_prev <= obj_Val_cur
        
        prev_loss = curr_loss;
        
        %% OWL of ak
        for k=1:K_greedy
            % Co-ordinate descent on a_k
            %             tic
            Wtemp1 = zeros(p-1,p-1);
            for k_iter=1:K_greedy
                Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
            end
            Wtemp1 = Wtemp1-diag(diag(Wtemp1));
            
            w_beta_penalty = sum(lambdaVect_beta.*abs(w_beta));
            
            w_ak_penalty = 0;
            for kk=1:K_greedy
                if kk~=k
                    w_ak_penalty = w_ak_penalty + lambda_akak*sum(abs(w_ak{kk}));
                end
            end
            w_penalty = w_beta_penalty+w_ak_penalty;
            
            %             fprintf('\nCo-ordinate descent over a_k at k = %d\n',k);
            lossArgs_ak{k} = {w_beta,X,y,Wtemp1, w_penalty};
            w_ak{k} = optimFunc_akbk(loss_ak, w_ak{k},lambdaVect_ak,options,lossArgs_ak{k}{:});
            %             toc
            % %             if norm(w_ak{k},2) > 0
            % %                 w_ak{k} = w_ak{k}/norm(w_ak{k},2);
            % %             end
        end
        
        Wtemp1 = zeros(p-1,p-1);
        for k_iter=1:K_greedy
            Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
        end
        Wtemp1 = Wtemp1-diag(diag(Wtemp1));
        
        %% Find Loss Function Value for current ak and previous beta
        Xbeta = X*w_beta;
        XWX_pred = zeros(1,n);
        for i=1:n
            XWX_pred(i) = X(i,(2:p))*Wtemp1*X(i,(2:p))';
        end
        
        Xbeta_XWX = (Xbeta+XWX_pred');
        %         y_pred_temp = 1./(1+exp(-Xbeta_XWX));
        %         y_pred = (y_pred_temp>=0.5)*2-1;
        ypred_Xbeta_XWX = y.*(Xbeta_XWX);
        
        loss_obj_ak = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta_XWX])); % log loss
        %         loss_obj_ak = loss_obj_ak/n;
        w_ak_penalty = 0;
        fprintf('Loss function for ak is %12.5e witk K rank =%d at itr= %d\n',loss_obj_ak,K_greedy,iterations);
        
        
        if best_loss_ak > loss_obj_ak
            best_loss_ak = loss_obj_ak;
            best_ak = w_ak;
            best_beta_ak = w_beta;
            Krank_best_ak = K_greedy;
            best_itr_ak = iterations;
        end;
        
        clear Xbeta Xbeta_XWX y_pred y_pred_temp ypred_Xbeta_XWX
        
        %% OWL for beta
        Wtemp1 = zeros(p-1,p-1);
        for k_iter=1:K_greedy
            Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
        end
        Wtemp1 = Wtemp1-diag(diag(Wtemp1));
        % %         Co-ordinate descent on beta
        w_ak_penalty = 0;
        for kk=1:K_greedy
            w_ak_penalty = w_ak_penalty + lambda_akak*sum(abs(w_ak{kk}));
        end
        
        lossArgs_beta = {X,y,Wtemp1, w_ak_penalty};
        w_beta = optimFunc_beta(loss_beta,w_beta,lambdaVect_beta,options,lossArgs_beta{:});
        
        %% Find Loss Function Value using beta only
        Xbeta = X*w_beta;
        ypred_Xbeta = (Xbeta);
        loss_obj_beta = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta])); % log loss
        %         loss_obj_beta = loss_obj_beta/n;
        fprintf('Loss function for beta only is %12.5e witk K rank =%d at itr=%d\n',loss_obj_beta,K_greedy,iterations);
        
        clear Xbeta ypred_Xbeta
        
        if best_loss_betaOnly > loss_obj_beta;
            best_loss_betaOnly = loss_obj_beta;
            best_betaOnly = w_beta;
            Krank_best_betaOnly = K_greedy;
            best_itr_betaOnly = iterations;
        end
        
        %% Calculating Objective functions after convergence of OWL
        Wtemp1 = zeros(p-1,p-1);
        for k_iter=1:K_greedy
            Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
        end
        Wtemp1 = Wtemp1-diag(diag(Wtemp1));
        
        Xbeta = X*w_beta;
        for i=1:n
            XWX_pred(i) = X(i,(2:p))*Wtemp1*X(i,(2:p))';
        end
        Xbeta_XWX = (Xbeta+XWX_pred');
        % %         y_pred_temp = 1./(1+exp(-Xbeta_XWX));
        % %         y_pred = (y_pred_temp>=0.5)*2-1;
        ypred_Xbeta_XWX = y.*(Xbeta_XWX);
        
        nll2 = sum(mylogsumexp([zeros(n,1) -ypred_Xbeta_XWX]));
        %         nll2 = nll2/n;
        
        w_ak_penalty = 0;
        for k=1:K_greedy
            w_ak_penalty = w_ak_penalty + lambda_akak*sum(abs(w_ak{k}));
        end
        obj_Val = nll2 + lambda_beta*sum(abs(w_beta)) + w_ak_penalty;
        obj_Val_cur = obj_Val;
        
        if obj_Val_prev ~= 0;
            criterionObjective = abs(obj_Val_cur-obj_Val_prev)/obj_Val_prev;
        else
            criterionObjective = 0;
        end
        
        fprintf('Curr. obj. val = %12.5e, Prev. obj. val = %12.5e, Criterion = %12.5e, Iteration = %d\n',obj_Val_cur, obj_Val_prev, criterionObjective, iterations);
        obj_Val_prev = obj_Val_cur;
        iterations = iterations + 1;
      
    end
    
    Loss_val = [Loss_val, loss_obj_ak];
    Loss_beta_val = [Loss_beta_val, loss_obj_beta];
    
    %% Display the Previous and Current Objective val.
    fprintf('Curr. obj. = %12.5e, Prev. obj. = %12.5e, Curr. Los. = %12.5e, Prev. Los = %12.5e, Criterion = %12.5e, Itr = %d\n',obj_Val_cur, obj_Val_prev, curr_loss,prev_loss, criterionObjective, iterations);
    %% Check conditions for greedy optimization of K
    if K_greedy == 1
        obj_Val_ak_prev = nll2;
        obj_Val_ak_cur = nll2;
        %         obj_Val_ak_prev = loss_obj_ak;
        %         obj_Val_ak_cur = loss_obj_ak;
        if init_nll > nll2
            fprintf('***************************************************\n');
            fprintf('NLL at initialization (%12.5e) > NLL val at new ak{1} (%12.5e)...continuing\n',init_nll, nll2);
            fprintf('***************************************************\n');
        else
            fprintf('***************************************************\n');
            fprintf('NLL at initialization (%12.5e) < NLL val at new ak{1} (%12.5e)...continuing.. add ak\n',init_nll, nll2);
            fprintf('***************************************************\n');
        end
    else
        fprintf('***************************************************\n');
        fprintf('Added new ak, since Cur ak Obj. (%12.5e) <= Prev. ak Obj. (%12.5e)\n',obj_Val_ak_cur,obj_Val_ak_prev);
        fprintf('***************************************************\n');
        obj_Val_ak_prev = obj_Val_ak_cur;
        obj_Val_ak_cur = nll2;
        %         obj_Val_ak_cur = loss_obj_ak;
    end
    
    if obj_Val_ak_cur > obj_Val_ak_prev
        fprintf('***************************************************\n');
        fprintf('NLL of cur ak (%12.5e) > NLL at prev ak(%12.5e)...exiting\n',obj_Val_ak_cur,obj_Val_ak_prev);
        fprintf('***************************************************\n');
        w_ak{K_greedy} = 0;
        K_greedy = K_greedy -1;
        break;
    end
    K_greedy = K_greedy + 1;
    w_ak{K_greedy} = ones(1,p-1)';
    w_ak{K_greedy} = w_ak{K_greedy}/norm(w_ak{K_greedy},2);
  
end

% % % % figure,
% % % % subplot(211),plot(Loss_val),title('Loss ak');
% % % % subplot(212),plot(Loss_beta_val), title('Loss \beta');


Wtemp1 = zeros(p-1,p-1);
for k_iter=1:K_greedy
    Wtemp1 = Wtemp1 + w_ak{k_iter}*w_ak{k_iter}';
end
Wtemp1 = Wtemp1-diag(diag(Wtemp1));

minVal_ak = best_loss_ak;
minVal_beta = best_loss_betaOnly;

if init_nll < minVal_ak & init_nll < minVal_beta
    fprintf('***************************************************\n');
    fprintf('Best loss is at initialization\n');
    fprintf('Loss beta only = %12.5e, init loss = %12.5e, best ak loss = %12.5e\n',minVal_beta,init_nll,minVal_ak);
    fprintf('***************************************************\n');
    
    beta = beta_init;
    W = 0*Wtemp1;
    ak = 0;
    K_rank = 0;
elseif minVal_ak < minVal_beta
    fprintf('***************************************************\n');
    fprintf('Best loss is with ak and prev. beta\n');
    fprintf('Loss beta only = %12.5e, init loss = %12.5e, best ak loss = %12.5e, Krank = %d, itr =%d\n',minVal_beta,init_nll,minVal_ak, Krank_best_ak, best_itr_ak);
    fprintf('***************************************************\n');
    
    beta = best_beta_ak;
    ak = best_ak;
    K_rank = Krank_best_ak;
    Wtemp1 = zeros(p-1,p-1);
    for k_iter=1:K_rank
        Wtemp1 = Wtemp1 + ak{k_iter}*ak{k_iter}';
    end
    Wtemp1 = Wtemp1 - diag(diag(Wtemp1));
    W = Wtemp1;
else
    fprintf('***************************************************\n');
    fprintf('Best loss is with beta only (no ak terms included)\n');
    fprintf('Loss beta only = %12.5e, init loss = %12.5e, best ak loss = %12.5e, Krank = %d, itr = %d\n',minVal_beta,init_nll,minVal_ak, Krank_best_betaOnly, best_itr_betaOnly);
    fprintf('***************************************************\n');
    beta = best_betaOnly;
    W = 0*Wtemp1;
    ak = 0;
    K_rank = Krank_best_betaOnly;
end

