function [W,beta,ak,S_all, K_rank,beta_init, err_init] = OWD_LogisticReg_OD_multi(X_tr, Y_tr, X_te, Y_te, W_org, beta_org, beta_param, ak_param, top_num,top_num_b, opts)
% % % % Orthant-Wise Learning Program for logistic regression model
% % % % Inputs: X_tr, Y_tr, best_param, opts. X_tr, Y_tr ->training data. best_param -> best parameter found from CV. opts -> optimization options declared in the main function
% % % % Outputs: W, beta, ak, K_rank -> They correspond to Estimated W (Weight matrix), estimated beta, estimated ak and estimated K rank.
% Modified by Yizhe Zhang at NEC Labs, Princeton, 6,10,2016 (Include sk, and minor changes)
% Modified by Yizhe Zhang at NEC Labs, Princeton, 6,28,2016 (add multinomial regression support)


%% initialize s


X = cell2mat(X_tr);
y = cell2mat(Y_tr);

[n,p] = size(X);
nClass = max(y);

for cc = 1:nClass
    K_greedy{cc} =1;
    S_all{cc} = ones(K_greedy{cc},1);
end

%% Use High Order Logistic Regression loss function
% loss_beta = @LogisticLoss_highorder_beta_up;
% loss_ak = @LogisticLoss_highorder_akak_up;

loss_beta = @LogisticLoss_highorder_beta_Wnew_multi;
loss_ak = @LogisticLoss_highorder_akak_Wnew_multi;

%% OWL functions for beta and ak updates
% optimFunc_beta = @L1GeneralOrthantWise_higherorder_beta;
% optimFunc_akbk = @L1GeneralOrthantWise_higherorder_akak;

optimFunc_beta = @L1GeneralProjectedSubGradient;
optimFunc_ak = @L1GeneralProjectedSubGradient;

%% Set lambda (higher values yield more regularization/sparsity)
lambda_beta = beta_param;
lambda_ak = ak_param;

% Penalize all variables except bias
lambda_akak = lambda_ak;
lambda_beta = lambda_beta;
lambdaVect_ak = [lambda_akak*ones(p-1,1)];
lambdaVect_beta = [zeros(1,nClass); lambda_beta*ones(p-1,nClass)];

%% Set all beta variables to be initially 0
w_init = zeros(p,nClass);
w_beta = w_init;

% %% Initialize beta using Lasso or L1LogReg
% %% Initialize beta using OLS or Lasso
% lambda_L1logreg = [0.01, 0.1, 1, 10, 20, 30, 50];
% %lambda_L1logreg = [0.01];
% for ii = 1:length(lambda_L1logreg)
%     [L1LogRegpred{ii}, L1LogRegerr(ii), L1LogRegprob{ii}, L1Logregsw{ii}, L1LogRegbiase{ii}] = L1LogReg_TrnTst(X_tr{1}, Y_tr{1}, X_te{1}, Y_te{1},lambda_L1logreg(ii), 'nll');
% end
% err_L1LogReg = min(L1LogRegerr);
% [val,idx] = min(L1LogRegerr);
% beta_L1LogReg = L1Logregsw{idx};
% w_beta = full(beta_L1LogReg);
% beta_init = w_beta;
% err_init = err_L1LogReg;


%% Calculate Objective function at initialization
Wtemp1 = cell(nClass,1);
for cc = 1:nClass
    Wtemp1{cc} = zeros(p-1,p-1);
    if cc == 1; continue; end
    for k=1:K_greedy{cc}
        w_ak{cc}{k} = 1e0*ones(1,p-1)';
        w_ak{cc}{k} = w_ak{cc}{k}/norm(w_ak{cc}{k},2);
        Wtemp1{cc} = Wtemp1{cc} + w_ak{cc}{k}*w_ak{cc}{k}';
    end
    Wtemp1{cc} = Wtemp1{cc} - diag(diag(Wtemp1{cc}));
end


nll = find_nll(w_beta,Wtemp1,X,y, nClass);
% nll = nll/n;

w_ak_penalty = 0;
init_obj_Val = nll + lambda_beta*sum(abs(w_beta)) + w_ak_penalty;
init_nll = nll;

fprintf('Loss fun. val. at initialization is %12.5e and Obj. val is %12.5e\n', init_nll, init_obj_Val);
fprintf('***************************************************\n');




%% %%%%%%%%%%%%%%%%%%%% Run Methods %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('\nAlgorithm: Alternating Optimization Using OWL...\n');
this_class_end = false(nClass,1);
W_old = Wtemp1;
obj_Val_ak = nll;
%this_class_end(cc) = ~(all(w_ak{cc}{K_greedy{cc}}) ~= 0 && K_greedy{cc} <=10);
best = struct();
while true
    for cc = 2:nClass      
        if ~this_class_end(cc)
            %if exit_flag{cc} == 1; continue; end;
            [K_greedy{cc}, w_ak{cc}, w_beta,S_all{cc},obj_Val_ak, best, exit_flag(cc)] = opt_routine(nClass, cc, K_greedy{cc}, w_ak{cc}, w_beta,S_all{cc}, obj_Val_ak,best,lambda_akak, lambda_beta,lambdaVect_ak, lambdaVect_beta,loss_ak,loss_beta, optimFunc_ak, optimFunc_beta, opts,X,y,W_old,w_ak,W_org, beta_org,top_num,top_num_b);
            W_old{cc} = cal_Wtemp(w_ak{cc},p,K_greedy{cc},S_all{cc},-1);
            this_class_end(cc) = exit_flag(cc) || (~(all(w_ak{cc}{K_greedy{cc}}) ~= 0 && K_greedy{cc} <=10));
        end
    end
    if all(this_class_end(2:end))
        break  
    end
end

% % % % figure,
% % % % subplot(211),plot(Loss_val),title('Loss ak');
% % % % subplot(212),plot(Loss_beta_val), title('Loss \beta');

for cc = 2:nClass
    Wtemp1{cc} = cal_Wtemp(w_ak{cc},p,K_greedy{cc},S_all{cc},-1);
end

minVal_ak = best_loss_ak;
minVal_beta = best_loss_betaOnly;

if init_nll < minVal_ak && init_nll < minVal_beta
    fprintf('***************************************************\n');
    fprintf('Best loss is at initialization\n');
    fprintf('Loss beta only = %12.5e, init loss = %12.5e, best ak loss = %12.5e\n',minVal_beta,init_nll,minVal_ak);
    fprintf('***************************************************\n');
    
    beta = beta_init;
    for cc = 2:nClass
        W{cc} = 0*Wtemp1{cc};
    end
    ak = 0;
    K_rank = 0;
elseif minVal_ak < minVal_beta
    fprintf('***************************************************\n');
    fprintf('Best loss is with ak and prev. beta\n');
    fprintf('Loss beta only = %12.5e, init loss = %12.5e, best ak loss = %12.5e, Krank = %d, itr =%d\n',minVal_beta,init_nll,minVal_ak, Krank_best_ak, best_itr_ak);
    fprintf('***************************************************\n');
    
    beta = best_beta_ak;
    ak = best_ak;
    K_rank = Krank_best_ak;
    for cc = 2:nClass
        W{cc} = cal_Wtemp(ak{cc},p,K_greedy{cc},S_all{cc},-1);
    end
else
    fprintf('***************************************************\n');
    fprintf('Best loss is with beta only (no ak terms included)\n');
    fprintf('Loss beta only = %12.5e, init loss = %12.5e, best ak loss = %12.5e, Krank = %d, itr = %d\n',minVal_beta,init_nll,minVal_ak, Krank_best_betaOnly, best_itr_betaOnly);
    fprintf('***************************************************\n');
    beta = best_betaOnly;
    for cc = 2:nClass
        W{cc} = 0*Wtemp1{cc};
    end
    ak = 0;
    K_rank = Krank_best_betaOnly;
end





