function [X, Y, X_original, Y_original] = RCCdata_generate_logreg(path_name, case_number)
%% RCC Data load and preprocess for linear regression with high order feature interactions
% % % % % This program loads RCC data and then labels the outcomes based on the experiment case number.
% % % % % Inputs: (path_name, case_number) ->  path_name, case_number. path_name is the full path of the location of the RCC dataset. 
% % % % %         path_name is a string, for eg. '/home/ml/spurushotham/FHIM Codes/RCC'. case_number is the classification experiment number for RCC dataset. 
% % % % %         If case_number = 1, then it is benign vs stage 1-4 classification experiment. Similarly, case_number =2 is benign, stage 1 vs stage 2-3 and 
% % % % %         case_number =3 is benign, stage 1,2 vs stage 3,4. maximum simulations is number of times the classification experiment needs to be run. 
% % % % % Outputs: [X, Y, X_original, Y_original] -> X is a design matrix, Y are outcomes, both are in cell format. X has an additional column of 1's for bias term. 
% % % % %         X_original and Y_orginial are same as X and Y but in matrix format, X_original does not have bias term column. 

%% Load RCC Data
cd_path = cd;
%cd('../Real_dataset/RCC');
cd(path_name)
load rcc
cd(cd_path); 

% Classification labels setup, Note: classes are hardcoded!!
Y_org = -1*ones(size(Y,1),3);
% Case 1: Classification of Benign from stages 1-4 samples
Y_org((1:40),1) = 1;
% Case 2: Classification of Benign and stage 1 samples from stages 2-4 samples
Y_org((1:141),2) = 1;
% Case 3: Classification of Benign, stages 1 and 2 samples from stages 3-4 samples
Y_org((1:158),3) = 1;

%% Data Splitting
X_original = X;
Y_original = Y_org;

if case_number ==1
    Y = Y_org(:,1);
elseif case_number ==2
    Y = Y_org(:,2);
elseif case_number ==3
    Y = Y_org(:,3);
else
    error('Wrong experiment case number .... exiting with this error message');
end

%% Preprocessing data
[nInstances, pVars] = size(X);
X_temp(:,(2:pVars+1)) = X;
X_temp(:,1) = 1;
clear X;

X = X_temp;
clear X_temp;

Y1{1} = Y;
x_temp{1} = X;
clear X;
clear Y;
Y = Y1;
X = x_temp;
clear  x_temp;
clear Y1;

end