function avg_rocscore = eval_MTL_logistic_roc_highorder (Y, X, W, beta)
%% FUNCTION for avg_rocscore for logistic regression
%
%% FORMULATION
%   y_predicted = 1/exp(-(XWX+Xbeta));
%   y_predicted = (y_predicted>=0.5)*2-1;
%   roc_score = roc(y_predicted, Y_original);
%
%% INPUT
%   X: {n * d} * t - input matrix. t is task number, for single task it's
%   1.
%   Y: {n * 1} * t - output matrix.
%   W: Weight matrix
%   beta : beta vector
%
%% OUTPUT
%   avg_rocscore - Average ROC score on test data
%% Last modified on Aug 2, 2013.
% Author: Sanjay Purushotham at NECLA, Princeton

task_num = length(X);

total_sample = 0;
for t = 1: task_num
    [n p] = size(X{t}); % get number of instances and predictors
    X1 = X{t};
    y= Y{t};
    Xbeta = X1*beta;
    for i=1:n
        XWX_pred(i) = X1(i,(2:p))*W*X1(i,(2:p))';
    end
    Xbeta_XWX = (Xbeta+XWX_pred');
    
    y_pred = 1./(1+exp(-Xbeta_XWX));
    y_pred = (y_pred>=0.5)*2-1;
    roc_score(t) = roc(y_pred,Y{t},'no_figure'); 
end
avg_rocscore = mean(roc_score); 
end
