function avg_error = eval_MTL_logregerror_highorder_new (Y, X, W, beta)
%% FUNCTION for misclassification error for logistic regression
%   computation of error given a specific model.
%   the value the lower the better.
%
%% FORMULATION
%   y_predicted = 1/exp(-(XWX+Xbeta));
%   y_predicted = (y_predicted>=0.5)*2-1;
%   sign(y_predicted ~= y_original)/n
%
%% INPUT
%   X: {n * d} * t - input matrix
%   Y: {n * 1} * t - output matrix
%   W: Interaction Weight matrix
%   beta: main terms (beta vector)
%
%% OUTPUT
%   avg_error - Average classification error on the test data
%% Last modified on Aug 2, 2013.
% Author: Sanjay Purushotham at NECLA, Princeton
 
task_num = length(X);
error = 0;

total_sample = 0;
for t = 1: task_num
    [n p] = size(X{t}); % get number of instances and predictors
    X1 = X{t};
    y= Y{t};
    Xbeta = X1*beta;
    for i=1:n
        XWX_pred(i) = X1(i,(2:p))*W*X1(i,(2:p))';
    end
    Xbeta_XWX = (Xbeta+XWX_pred');
    
    y_pred = 1./(1+exp(-Xbeta_XWX));
    y_pred = (y_pred>=0.5)*2-1;
    error = error + sum(y~=y_pred);
    total_sample = total_sample + length(y_pred); 
end
avg_error = error./total_sample;
end
