function [X, Y, beta_org, W_org, s_k, epsilon] = syndata_generate_logreg_multi(nInstances, nClass, pVars, K_interactions, beta_sparsity, ak_sparsity,setting)
%% Synthetic Data generation for linear regression with high order feature interactions
% % % % This program generates synthetic data for the FHIM logistic regression model. 
% % % % Inputs: (nInstances, pVars, K_interactions, beta_sparsity, ak_sparsity) -> nInstances is the number of instances that needs to be generated (n). pVars - number of variables/features (p). K_interactions is rank K of W. beta_sparsity is number of non-zeros in beta. ak_sparsity is number of non-zeros in ak. 
% % % % Outputs: (X, Y, beta_org, W_org, epsilon) -> X is design matrix (n X p), Y is outcomes (n X 1), beta_org is the orginial beta vector (p X 1), W_org is the original W matrix (p X p), epsilon is the noise vector ( n X 1).
% Modified by Yizhe Zhang at NEC Labs, Princeton, 6,10,2016 (Include sk, and minor changes)
% 6,21,2016 (Add multinomial regression support)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sparsity_ak = ak_sparsity/pVars;
Kn_ak = sparsity_ak*pVars;

% % Kn_beta = beta_sparsity;
% % I_k =[ones(1,Kn_beta) zeros(1,pVars-Kn_beta)];
% % beta = 1.0*randn(1,pVars).*I_k; %% regression co-efficients

sparsity_beta = beta_sparsity/pVars;
beta = 1.0*sprandn(pVars+1,nClass, sparsity_beta);
beta(:,1) = zeros(pVars+1,1);

epsilon = 0.1*randn(1,nInstances)'; %% noise

% X = 2*randn(nInstances, pVars); %% design matrix
X = [ones(nInstances,1) randn(nInstances, pVars)]; %% design matrix

%%%%%%%%% W = sum_k s_k * a_k*a_k^T %%%%%%%%%%%
for cc = 1:nClass
    W_temp{cc} = zeros(pVars, pVars);
    if cc == 1; continue; end
    if ~setting.isPosDef
        s_k{cc} = (rand(K_interactions,1)>0.5)*2-1;
    else
        s_k{cc} = ones(K_interactions,1);
    end

    for k=1:K_interactions
        a_k{cc} = 1*sprandn(1,pVars,sparsity_ak);
    % %     a_k = a_k/norm(a_k,2); %% ||a_k||_2 = 1
    %    ak{k} = a_k;
        W_temp{cc} = W_temp{cc} + s_k{cc}(k)*(a_k{cc}'*a_k{cc});
    end
    W_temp{cc} = W_temp{cc} - diag(diag(W_temp{cc}));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Logistic Regression with higher order feature interaction %%%%%%%%%%%%

flipFactor = 0.00;
flipPos = rand(nInstances,1) < flipFactor;

XWX = zeros(nInstances,nClass);
for cc = 1:nClass
    for i=1:nInstances
        XWX(i,cc) = X(i,(2:pVars+1))* W_temp{cc} * X(i,(2:pVars+1))';
    end
end
softmax = @(x) bsxfun(@rdivide,exp(-x),sum(exp(-x),2)); %n*c
yprob = softmax(X*beta+ XWX) ;
y = zeros(nInstances,1);
for i=1:nInstances
    y(i) = randsample(1:nClass,1,true,yprob(i,:));
end
% bar(X*beta');hold on;bar(XWX');
%y(flipPos) = -y(flipPos); % Adding noise by flipping the sign of y

X_org = X;
y_org = y;

Y_pos = sum(y_org==1);
Y_neg = sum(y_org==-1);
Y_diff = Y_pos - Y_neg ;
fprintf('Y class 1 = %d, Y class -1 = %d\n',Y_pos,Y_neg);
s_k
% % % % Checking if the data is balanced or not 
% % % % flag = 1;
% % % % if (abs(Y_diff) >=100)
% % % %     fprintf('Unbalanced dataset, exiting \n');
% % % %     flag = 0;
% % % % else
% % % %     fprintf('balanced dataset, continuing \n');
% % % % end

beta_org = beta;
W_org = W_temp;
X = X;
Y = y';
end